# Copyright 2015 Rodrigo Fernandes da Costa

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

class Question
  def initialize(text, answer)
    @text, @answer = text, answer
  end

  def text; @text; end
  def answer; @answer; end

  def check guess
    guess.strip == @answer.strip
  end

  def self.load fileName
    IO.readlines(fileName).map do |line|
      text, question = line.scan(/(.*\? )(.*)/)[0]
      Question.new(text, question)
    end
    rescue Exception => error
      User.warn error.message
      raise error
  end
end

Memory = Struct.new(:question, :strength, :index)

class Memorization
  def initialize questions
    @list = Array.new(questions.size) do |i|
      Memory.new(questions[i], 0, i)
    end
    @corrects = 0
    @wrongs = 0
  end

  def self.load
    file = File.new("Save")
    Marshal.load(file)
    rescue
      return nil
    ensure
      file.close if file
  end

  def save
    file = File.new("Save", "w")
    Marshal.dump(self, file)
    rescue Exception => error
      User.warn error.message
      User.warn "Session won't be saved"
      gets
    ensure
      file.close if file
  end

  def memorize
    loop do
      memory = @list.min_by &:index
      correct = yield memory.question
      @corrects += 1 if correct
      @wrongs += 1 unless correct
      if correct
        memory.strength += 1
      else
        memory.strength = 0
      end
      memory.index += memory.strength * 2
      @list.each do |i|
        i.index -= 1  if i.index <= memory.index and i != memory
      end
    end
  end

  def report
    count = @list.count{|i| not i.strength.zero?}
    puts "#{count} of #{@list.size} itens were visited."
    percentage = @corrects * 100 / (@wrongs + @corrects).to_f
    percentage = 0 if (@wrongs + @corrects).zero?
    puts "%.0f%% correct answers." % percentage
  end
end

class User
  def self.warn warning
    puts warning
  end

  def self.getQuestions
    print "File to load questions from: "
    file = gets.chomp.delete '"'
    Question.load(file).shuffle
    rescue; retry
  end

  def self.showInstructions
    puts "Commands: reset, quit, report"
  end

  def self.memorize memorization
    command = memorization.memorize do |question|
      print question.text
      answer = gets.chomp
      break answer if answer == "quit"
      break answer if answer == "reset"
      if answer == "report"
        memorization.report
        redo
      end
      if question.check answer
        next true
      else
        puts "The right answer is: " + question.answer
        next false
      end
    end
    User.memorize Memorization.new(User.getQuestions) if command == "reset"
    memorization.save if command == "quit"
  end
end

User.showInstructions

User.memorize((Memorization.load or Memorization.new(User.getQuestions)))
